import os

from pydantic.v1 import BaseSettings


class AbstractSettings(BaseSettings):
    DATABASE = {
        "driver": "sqlite",
        "name": "sqlite.db",
        "user": "",
        "password": "",
        "host": "",
        "port": "",
    }

    DATABASE_URL: str = (
        f"{DATABASE['driver']}:///{DATABASE['name']}"
        if DATABASE["driver"] == "sqlite"
        else f"{DATABASE['driver']}://{DATABASE['user']}:{DATABASE['password']}@{DATABASE['host']}:{DATABASE['port']}/{DATABASE['name']}"
    )

    class Config:
        env_file = os.environ.get("SETTINGS_ENV", ".env")



from sqlalchemy.orm import Session


class Repository:
    DB_MODEL = None

    def __init__(self, db: Session):
        self.db = db

    def get_by_pk(self, pk: int | str):
        instance = self.db.query(self.DB_MODEL).filter(self.DB_MODEL.pk == pk).first()
        return instance

    def get_by_hash_id(self, hash_id: str):
        instance = self.db.query(self.DB_MODEL).filter(self.DB_MODEL.hash_id == hash_id).first()
        return instance



import abc
import os
import subprocess
from abc import ABC


class Command(ABC):
    name: str = None
    help: str = None

    @abc.abstractmethod
    def execute(self):
        raise NotImplementedError


def run_alembic(alembic_command: str):
    subprocess.run(alembic_command.split(" "))


def copy_file(file_path, new_file_path, **kwargs):
    with open(file_path, 'r', encoding='utf-8') as f:
        data = f.read().format(**kwargs)

    with open(new_file_path, 'w', encoding='utf-8') as f:
        f.write(data)
    print(f"✅ Created {file_path}")


def copy_files(source_dir, target_dir, **kwargs):
    for file_path in source_dir.rglob('*'):
        if file_path.is_file():
            relative_path = file_path.relative_to(source_dir)
            relative_path = relative_path.with_suffix(".py")
            new_file_path = target_dir / relative_path

            if os.path.isfile(new_file_path):
                print(f"⚠️  {file_path} already exists, skipping")
                continue

            new_file_path.parent.mkdir(parents=True, exist_ok=True)
            copy_file(file_path, new_file_path, **kwargs)



import os
from importlib import resources
from pathlib import Path

import typer

from fastapi_clean_archi.managements.commands.base import Command, copy_files, copy_file

template_dir = resources.files("fastapi_clean_archi.managements.templates")


class AddModule(Command):
    name = "add_module"
    help = "구조화된 모듈을 추가합니다."

    def execute(self, module_name=typer.Argument(...)):
        class_name = module_name.capitalize()
        module_name = module_name.lower()

        copy_files(source_dir=Path(f"{template_dir}/app_module"),
                   target_dir=Path(f"./app/modules/{module_name}"),
                   module_name=module_name,
                   module_class=class_name)

        if not os.path.exists("./app/modules/__init__.py"):
            copy_file(file_path=Path(f"{template_dir}/__init__.tmpl"),
                      new_file_path=Path("./app/modules/__init__.py"))




import importlib
import os

import typer
import uvicorn

from core.config import AbstractSettings
from fastapi_clean_archi.managements.commands.base import Command


class Runserver(Command):
    name = "runserver"
    help = "FastAPI server 가 실행됩니다."

    def execute(self,
                host_port=typer.Argument("localhost:8000", help="호스트와 포트를 지정합니다. 예: localhost:8000"),
                settings_env=typer.Option(".env", "--settings-env")):
        host, port = host_port.split(":")
        try:
            module = importlib.import_module("app.core.config")
            settings = module.settings
            print(settings.dict())
        except ModuleNotFoundError:
            raise ModuleNotFoundError("app.core.config 모듈을 찾을 수 없습니다. 프로젝트 구조를 확인해주세요.")

        os.environ.setdefault("SETTINGS_ENV", settings_env)
        uvicorn.run("main:app", host=host, port=int(port), reload=True)


import typer

from fastapi_clean_archi.managements.commands.base import Command, run_alembic


class MakeMigrations(Command):
    name = "makemigrations"
    help = "db migration 파일을 생성합니다."

    def execute(self, message=typer.Argument("auto_migration")):
        run_alembic(f"alembic revision --autogenerate -m {message}")



from fastapi_clean_archi.managements.commands.base import run_alembic, Command


class Migrate(Command):
    name = "migrate"
    help = "db migration 을 진행합니다."

    def execute(self):
        run_alembic("alembic upgrade head")



import argparse
import re
import secrets
from importlib import resources
from pathlib import Path

from fastapi_clean_archi.managements.commands.base import copy_files, copy_file, run_alembic

template_dir = resources.files("fastapi_clean_archi.managements.templates")


def edit_file(filename, remove_line_words, add_lines, line_number):
    path = Path(filename)
    lines = path.read_text().splitlines()

    for index in range(len(lines)):
        for x in remove_line_words:
            if lines[index].startswith(x):
                lines[index] = ""
    path.write_text("\n".join(lines[:line_number] + add_lines + lines[line_number:]))


def create_core():
    secret_key = secrets.token_urlsafe(32)
    copy_files(source_dir=Path(f"{template_dir}/core"),
               target_dir=Path("./app/core"),
               secret_key=secret_key)

    copy_file(file_path=Path(f"{template_dir}/__init__.tmpl"),
              new_file_path=Path("./app/__init__.py"))

    copy_file(file_path=Path(f"{template_dir}/main.tmpl"),
              new_file_path=Path("./main.py"))

    copy_file(file_path=Path(f"{template_dir}/pytest.tmpl"),
              new_file_path=Path("./pytest.ini"))

    copy_file(file_path=Path(f"{template_dir}/manage.tmpl"),
              new_file_path=Path("./manage.py"))
    run_alembic("alembic init migrations")

    edit_file("./alembic.ini", remove_line_words=["sqlalchemy.url"], add_lines=[], line_number=0)
    edit_file("./migrations/env.py", remove_line_words=["target_metadata = "], add_lines=[
        "",
        "from app.core.config import settings",
        "config.set_main_option(\"sqlalchemy.url\", settings.DATABASE_URL)",
        "",
        "import os, importlib",
        "modules = os.listdir(\"app/modules\")",
        "for module in modules:",
        "    try:",
        "        importlib.import_module(f\"app.modules.{module}.infrastructure.models\")",
        "    except ModuleNotFoundError:",
        "        continue",
        "",
        "from fastapi_clean_archi.core.db.base import BaseModel",
        "target_metadata = BaseModel.metadata"
        "",
    ], line_number=19)



from passlib.context import CryptContext

pwd_context = CryptContext(
    schemes=["argon2", "bcrypt"],
    deprecated="auto",
)


def hash_password(password: str) -> str:
    return pwd_context.hash(password)


def verify_password(plain_password: str, hashed_password: str) -> bool:
    return pwd_context.verify(plain_password, hashed_password)



import uuid

from sqlalchemy import Column, Integer, DateTime, func, String, event
from sqlalchemy.orm import as_declarative, declarative_base, declared_attr, Session

Base = declarative_base()


@as_declarative()
class BaseModel(Base):
    __abstract__ = True

    hash_id = Column(String(100), nullable=False, unique=True)
    created_at = Column(DateTime(timezone=True), server_default=func.now(), nullable=False)
    updated_at = Column(DateTime(timezone=True), server_default=func.now(), onupdate=func.now(), nullable=False)

    @declared_attr
    def __tablename__(cls):
        meta = getattr(cls, "Meta", None)
        if meta and hasattr(meta, "db_table"):
            return meta.db_table
        return cls.__name__.lower()

    @declared_attr
    def pk(cls):
        for attr in cls.__dict__.values():
            if isinstance(attr, Column) and attr.primary_key:
                return Column("id", Integer, index=True, unique=True, autoincrement=True)
        return Column("id", Integer, primary_key=True, index=True, unique=True, autoincrement=True)


@event.listens_for(Session, "before_flush")
def generate_hash_id(session, flush_context, instances):
    for instance in session.new:
        if instance.hash_id is None:
            if getattr(instance, "user_id"):
                hash_id = str(uuid.uuid3(uuid.NAMESPACE_OID, f"{instance.user_id}_{instance.pk}").hex)
            else:
                hash_id = str(uuid.uuid4().hex)
            instance.hash_id = hash_id
